@@ -26,7 +26,6 @@
 
 #include <config.h>
 #include <stdio.h>
-#include <unicode.h>
 #include <ctype.h>
 #include <math.h>
 #include <string.h>
@@ -37,6 +36,7 @@
 #include <gtk/gtkinvisible.h>
 #include <gtk/gtksignal.h>
 #include <gdk/gdkkeysyms.h>
+#include <gtk/gtkwidget.h>
 #include <libgnomeui/gnome-canvas.h>
 #include <libgnomeui/gnome-canvas-rect-ellipse.h>
 #include "e-cell-text.h"
@@ -69,6 +69,7 @@ enum {
 	ARG_BOLD_COLUMN,
 	ARG_COLOR_COLUMN,
 	ARG_EDITABLE,
+	ARG_BG_COLOR_COLUMN
 };
 
 
@@ -313,6 +314,7 @@ static void
 ect_cancel_edit (ECellTextView *text_view)
 {
 	ect_stop_editing (text_view);
+	e_table_item_leave_edit (text_view->cell_view.e_table_item_view);
 }
 
 /*
@@ -393,6 +395,7 @@ ect_unrealize (ECellView *ecv)
 
 	if (text_view->font)
 		e_font_unref (text_view->font);
+	text_view->font = NULL;
 	
 	if (text_view->stipple)
 		gdk_bitmap_unref (text_view->stipple);
@@ -692,6 +695,27 @@ ect_draw (ECellView *ecell_view, GdkDrawable *drawable,
 	gdk_gc_set_clip_rectangle (fg_gc, NULL);
 }
 
+
+/*
+ * Get the background color
+ */
+static gchar *
+ect_get_bg_color(ECellView *ecell_view, int row)
+{
+	ECellText *ect = E_CELL_TEXT (ecell_view->ecell);
+	gchar *color_spec;
+
+	if (ect->bg_color_column == -1)
+		return NULL;
+
+	color_spec = e_table_model_value_at (ecell_view->e_table_model,
+	                                     ect->bg_color_column, row);
+
+	return color_spec;
+}
+					 			 		  			
+
+
 /*
  * Selects the entire string
  */
@@ -924,9 +948,13 @@ ect_height (ECellView *ecell_view, int model_col, int view_col, int row)
 	
 	font = text_view->font;
 
-	string = ect_get_text(ect, ecell_view->e_table_model, model_col, row);
-	value = e_font_height (font) * number_of_lines(string) + TEXT_PAD;
-	ect_free_text(ect, string);
+	if (row == -1) {
+		value = e_font_height (font) + TEXT_PAD;
+	} else {
+		string = ect_get_text(ect, ecell_view->e_table_model, model_col, row);
+		value = e_font_height (font) * number_of_lines(string) + TEXT_PAD;
+		ect_free_text(ect, string);
+	}
 
 	return value;
 }
@@ -1205,6 +1233,8 @@ ect_show_tooltip (ECellView *ecell_view,
 
 	canvas = e_canvas_new ();
 	gtk_container_add (GTK_CONTAINER (window), canvas);
+	GTK_WIDGET_UNSET_FLAGS (canvas, GTK_CAN_FOCUS);
+	GTK_WIDGET_UNSET_FLAGS (window, GTK_CAN_FOCUS);
 
 	max_width = 0.0;
 	for (lines = cell.breaks->lines, i = 0; i < cell.breaks->num_lines;
@@ -1322,6 +1352,10 @@ ect_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 		text->editable = GTK_VALUE_BOOL (*arg) ? TRUE : FALSE;
 		break;
 
+	case ARG_BG_COLOR_COLUMN:
+		text->bg_color_column = GTK_VALUE_INT (*arg);
+		break;
+
 	default:
 		return;
 	}
@@ -1352,6 +1386,10 @@ ect_get_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 		GTK_VALUE_BOOL (*arg) = text->editable ? TRUE : FALSE;
 		break;
 
+	case ARG_BG_COLOR_COLUMN:
+		GTK_VALUE_INT (*arg) = text->bg_color_column;
+		break;
+
 	default:
 		arg->type = GTK_TYPE_INVALID;
 		break;
@@ -1379,6 +1417,7 @@ e_cell_text_class_init (GtkObjectClass *object_class)
 	ecc->print_height = ect_print_height;
 	ecc->max_width = ect_max_width;
 	ecc->show_tooltip = ect_show_tooltip;
+	ecc->get_bg_color = ect_get_bg_color;
 
 	ectc->get_text = ect_real_get_text;
 	ectc->free_text = ect_real_free_text;
@@ -1396,6 +1435,8 @@ e_cell_text_class_init (GtkObjectClass *object_class)
 				 GTK_TYPE_INT, GTK_ARG_READWRITE, ARG_COLOR_COLUMN);
 	gtk_object_add_arg_type ("ECellText::editable",
 				 GTK_TYPE_BOOL, GTK_ARG_READWRITE, ARG_EDITABLE);
+	gtk_object_add_arg_type ("ECellText::bg_color_column",
+				 GTK_TYPE_INT, GTK_ARG_READWRITE, ARG_BG_COLOR_COLUMN);
 
 	if (!clipboard_atom)
 		clipboard_atom = gdk_atom_intern ("CLIPBOARD", FALSE);
@@ -1409,6 +1450,7 @@ e_cell_text_init (ECellText *ect)
 	ect->strikeout_column = -1;
 	ect->bold_column = -1;
 	ect->color_column = -1;
+	ect->bg_color_column = -1;
 	ect->editable = TRUE;
 }
 
@@ -1607,7 +1649,7 @@ _get_position_from_xy (CurrentCell *cell, gint x, gint y)
 	lines += j;
 	xpos = get_line_xpos (cell, lines);
 
-	for (p = lines->text; p < lines->text + lines->length; p = unicode_next_utf8 (p)) {
+	for (p = lines->text; p < lines->text + lines->length && g_unichar_validate (g_utf8_get_char (p)); p = g_utf8_next_char (p)) {
 		gint charwidth;
 
 		charwidth = e_font_utf8_char_width (font, cell->style, p);
@@ -1733,15 +1775,15 @@ _get_position (ECellTextView *text_view, ETextEventProcessorCommand *command)
 
 		if (edit->selection_end < 1) return 0;
 
-		p = unicode_previous_utf8 (cell->text, cell->text + edit->selection_end);
+		p = g_utf8_find_prev_char (cell->text, cell->text + edit->selection_end);
 
 		if (p == cell->text) return 0;
 
-		p = unicode_previous_utf8 (cell->text, p);
+		p = g_utf8_find_prev_char (cell->text, p);
 
 		while (p && p > cell->text) {
 			if (*p == '\n') return p - cell->text + 1;
-			p = unicode_previous_utf8 (cell->text, p);
+			p = g_utf8_find_prev_char (cell->text, p);
 		}
 
 		return 0;
@@ -1751,11 +1793,11 @@ _get_position (ECellTextView *text_view, ETextEventProcessorCommand *command)
 		length = strlen (cell->text);
 		if (edit->selection_end >= length) return length;
 
-		p = unicode_next_utf8 (cell->text + edit->selection_end);
+		p = g_utf8_next_char (cell->text + edit->selection_end);
 
-		while (*p) {
+		while (*p && g_unichar_validate (g_utf8_get_char (p))) {
 			if (*p == '\n') return p - cell->text;
-			p = unicode_next_utf8 (p);
+			p = g_utf8_next_char (p);
 		}
 
 		return p - cell->text;
@@ -1765,7 +1807,7 @@ _get_position (ECellTextView *text_view, ETextEventProcessorCommand *command)
 		length = strlen (cell->text);
 		if (edit->selection_end >= length) return length;
 
-		p = unicode_next_utf8 (cell->text + edit->selection_end);
+		p = g_utf8_next_char (cell->text + edit->selection_end);
 
 		return p - cell->text;
 
@@ -1773,7 +1815,7 @@ _get_position (ECellTextView *text_view, ETextEventProcessorCommand *command)
 
 		if (edit->selection_end < 1) return 0;
 
-		p = unicode_previous_utf8 (cell->text, cell->text + edit->selection_end);
+		p = g_utf8_find_prev_char (cell->text, cell->text + edit->selection_end);
 
 		if (p == NULL) return 0;
 
@@ -1784,12 +1826,12 @@ _get_position (ECellTextView *text_view, ETextEventProcessorCommand *command)
 		length = strlen (cell->text);
 		if (edit->selection_end >= length) return length;
 
-		p = unicode_next_utf8 (cell->text + edit->selection_end);
+		p = g_utf8_next_char (cell->text + edit->selection_end);
 
-		while (*p) {
-			unicode_get_utf8 (p, &unival);
-			if (unicode_isspace (unival)) return p - cell->text;
-			p = unicode_next_utf8 (p);
+		while (*p && g_unichar_validate (g_utf8_get_char (p))) {
+			unival = g_utf8_get_char (p);
+			if (g_unichar_isspace (unival)) return p - cell->text;
+			p = g_utf8_next_char (p);
 		}
 
 		return p - cell->text;
@@ -1798,18 +1840,18 @@ _get_position (ECellTextView *text_view, ETextEventProcessorCommand *command)
 
 		if (edit->selection_end < 1) return 0;
 
-		p = unicode_previous_utf8 (cell->text, cell->text + edit->selection_end);
+		p = g_utf8_find_prev_char (cell->text, cell->text + edit->selection_end);
 
 		if (p == cell->text) return 0;
 
-		p = unicode_previous_utf8 (cell->text, p);
+		p = g_utf8_find_prev_char (cell->text, p);
 
-		while (p && p > cell->text) {
-			unicode_get_utf8 (p, &unival);
-			if (unicode_isspace (unival)) {
-				return (unicode_next_utf8 (p) - cell->text);
+		while (p && p > cell->text && g_unichar_validate (g_utf8_get_char (p))) {
+			unival = g_utf8_get_char (p);
+			if (g_unichar_isspace (unival)) {
+				return (g_utf8_next_char (p) - cell->text);
 			}
-			p = unicode_previous_utf8 (cell->text, p);
+			p = g_utf8_find_prev_char (cell->text, p);
 		}
 
 		return 0;
@@ -2202,7 +2244,7 @@ number_of_lines (char *text)
 
 	if (!text) return 0;
 
-	for (p = text; *p; p = unicode_next_utf8 (p)) {
+	for (p = text; *p && g_unichar_validate (g_utf8_get_char (p)); p = g_utf8_next_char (p)) {
 		if (*p == '\n') num_lines++;
 	}
 	
@@ -2246,7 +2288,7 @@ split_into_lines (CurrentCell *cell)
 	linebreaks->lines = lines = g_new0 (struct line, linebreaks->num_lines);
 
 	len = 0;
-	for (p = text; *p; p = unicode_next_utf8 (p)) {
+	for (p = text; *p && g_unichar_validate (g_utf8_get_char (p)); p = g_utf8_next_char (p)) {
 		if (len == 0) lines->text = p;
 		if (*p == '\n') {
 			lines->length = p - lines->text;
