@@ -5,6 +5,10 @@
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gal/e-table/e-cell.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif /* __cplusplus */
+
 #define E_TABLE_COL_TYPE        (e_table_col_get_type ())
 #define E_TABLE_COL(o)          (GTK_CHECK_CAST ((o), E_TABLE_COL_TYPE, ETableCol))
 #define E_TABLE_COL_CLASS(k)    (GTK_CHECK_CLASS_CAST((k), E_TABLE_COL_TYPE, ETableColClass))
@@ -31,10 +35,11 @@ typedef struct {
 	GCompareFunc compare;
 	unsigned int is_pixbuf:1;
 	unsigned int selected:1;
-	unsigned int resizeable:1;
+	unsigned int resizable:1;
 	unsigned int sortable:1;
 	unsigned int groupable:1;
 	int          col_idx;
+	int          priority;
 
 	GtkJustification justification;
 
@@ -49,14 +54,19 @@ GtkType        e_table_col_get_type        (void);
 ETableCol     *e_table_col_new             (int col_idx, const char *text,
 					    double expansion, int min_width,
 					    ECell *ecell, GCompareFunc compare,
-					    gboolean resizable);
+					    gboolean resizable, int priority);
 ETableCol     *e_table_col_new_with_pixbuf (int col_idx, const char *text, 
 					    GdkPixbuf *pixbuf,
 					    double expansion, int min_width,
 					    ECell *ecell, GCompareFunc compare,
-					    gboolean resizable);
+					    gboolean resizable, int priority);
 void           e_table_col_destroy         (ETableCol *etc);
 
 
+#ifdef __cplusplus
+}
+#endif /* __cplusplus */
+
+
 #endif /* _E_TABLE_COL_H_ */
 
