@@ -51,6 +51,7 @@ enum {
 	RIGHT_CLICK,
 	CLICK,
 	KEY_PRESS,
+	START_DRAG,
 
 	TABLE_DRAG_BEGIN,
 	TABLE_DRAG_END,
@@ -68,6 +69,8 @@ enum {
 enum {
 	ARG_0,
 	ARG_LENGTH_THRESHOLD,
+	ARG_MODEL,
+	ARG_UNIFORM_ROW_HEIGHT,
 };
 
 static gint et_signals [LAST_SIGNAL] = { 0, };
@@ -117,12 +120,12 @@ static void et_drag_data_received(GtkWidget *widget,
 				  guint info,
 				  guint time,
 				  ETable *et);
-static gint e_table_drag_source_event_cb (GtkWidget      *widget,
-					  GdkEvent       *event,
-					  ETable         *table);
 
 static gint et_focus (GtkContainer *container, GtkDirectionType direction);
 
+static void scroll_off (ETable *et);
+static void scroll_on (ETable *et, gboolean down);
+
 static void
 et_disconnect_model (ETable *et)
 {
@@ -162,11 +165,13 @@ et_destroy (GtkObject *object)
 	if (et->group_info_change_id)
 		gtk_signal_disconnect (GTK_OBJECT (et->sort_info),
 				       et->group_info_change_id);
-
+	
 	if (et->reflow_idle_id)
 		g_source_remove(et->reflow_idle_id);
 	et->reflow_idle_id = 0;
 
+	scroll_off (et);
+
 	gtk_object_unref (GTK_OBJECT (et->model));
 	gtk_object_unref (GTK_OBJECT (et->full_header));
 	gtk_object_unref (GTK_OBJECT (et->header));
@@ -179,6 +184,9 @@ et_destroy (GtkObject *object)
 	if (et->header_canvas != NULL)
 		gtk_widget_destroy (GTK_WIDGET (et->header_canvas));
 
+	if (et->site != NULL)
+		e_table_drag_source_unset (et);
+
 	gtk_widget_destroy (GTK_WIDGET (et->table_canvas));
 
 	if (et->rebuild_idle_id) {
@@ -199,37 +207,40 @@ e_table_init (GtkObject *object)
 
 	GTK_WIDGET_SET_FLAGS (e_table, GTK_CAN_FOCUS);
 
-	gtk_table->homogeneous = FALSE;
+	gtk_table->homogeneous                      = FALSE;
 
-	e_table->sort_info = NULL;
-	e_table->group_info_change_id = 0;
-	e_table->reflow_idle_id = 0;
+	e_table->sort_info                          = NULL;
+	e_table->group_info_change_id               = 0;
+	e_table->reflow_idle_id                     = 0;
+	e_table->scroll_idle_id               = 0;
 
-	e_table->horizontal_draw_grid = 1;
-	e_table->vertical_draw_grid = 1;
-	e_table->draw_focus = 1;
-	e_table->cursor_mode = E_CURSOR_SIMPLE;
-	e_table->length_threshold = 200;
+	e_table->alternating_row_colors             = 1;
+	e_table->horizontal_draw_grid               = 1;
+	e_table->vertical_draw_grid                 = 1;
+	e_table->draw_focus                         = 1;
+	e_table->cursor_mode                        = E_CURSOR_SIMPLE;
+	e_table->length_threshold                   = 200;
+	e_table->uniform_row_height                 = FALSE;
 
-	e_table->need_rebuild = 0;
-	e_table->rebuild_idle_id = 0;
+	e_table->need_rebuild                       = 0;
+	e_table->rebuild_idle_id                    = 0;
 
-	e_table->horizontal_scrolling = FALSE;
+	e_table->horizontal_scrolling               = FALSE;
 
-	e_table->click_to_add_message = NULL;
+	e_table->click_to_add_message               = NULL;
 
-	e_table->drag_get_data_row = -1;
-	e_table->drag_get_data_col = -1;
-	e_table->drop_row = -1;
-	e_table->drop_col = -1;
-	e_table->site = NULL;
-	e_table->drag_source_button_press_event_id = 0;
-	e_table->drag_source_motion_notify_event_id = 0;
+	e_table->drag_get_data_row                  = -1;
+	e_table->drag_get_data_col                  = -1;
+	e_table->drop_row                           = -1;
+	e_table->drop_col                           = -1;
+	e_table->site                               = NULL;
 
-	e_table->sorter = NULL;
-	e_table->selection = e_table_selection_model_new();
-	e_table->cursor_loc = E_TABLE_CURSOR_LOC_NONE;
-	e_table->spec = NULL;
+	e_table->do_drag                            = 0;
+
+	e_table->sorter                             = NULL;
+	e_table->selection                          = e_table_selection_model_new();
+	e_table->cursor_loc                         = E_TABLE_CURSOR_LOC_NONE;
+	e_table->spec                               = NULL;
 }
 
 /* Grab_focus handler for the ETable */
@@ -302,8 +313,11 @@ sort_info_changed (ETableSortInfo *info, ETable *et)
 	gboolean will_be_grouped = e_table_sort_info_grouping_get_count(info) > 0;
 	if (et->is_grouped || will_be_grouped) {
 		et->need_rebuild = TRUE;
-		if (!et->rebuild_idle_id)
+		if (!et->rebuild_idle_id) {
+			gtk_object_destroy (GTK_OBJECT (et->group));
+			et->group = NULL;
 			et->rebuild_idle_id = g_idle_add_full (20, changed_idle, et, NULL);
+		}
 	}
 }
 
@@ -422,13 +436,17 @@ click_to_add_cursor_change (ETableClickToAdd *etcta, int row, int col, ETable *e
 static void
 group_cursor_change (ETableGroup *etg, int row, ETable *et)
 {
-	if (et->cursor_loc == E_TABLE_CURSOR_LOC_ETCTA && et->click_to_add) {
-		e_table_click_to_add_commit(E_TABLE_CLICK_TO_ADD(et->click_to_add));
-	}
+	ETableCursorLoc old_cursor_loc;
+
+	old_cursor_loc = et->cursor_loc;
+
 	et->cursor_loc = E_TABLE_CURSOR_LOC_TABLE;
 	gtk_signal_emit (GTK_OBJECT (et),
 			 et_signals [CURSOR_CHANGE],
 			 row);
+
+	if (old_cursor_loc == E_TABLE_CURSOR_LOC_ETCTA && et->click_to_add)
+		e_table_click_to_add_commit(E_TABLE_CLICK_TO_ADD(et->click_to_add));
 }
 
 static void
@@ -472,43 +490,30 @@ group_key_press (ETableGroup *etg, int row, int col, GdkEvent *event, ETable *et
 {
 	int return_val = 0;
 	GdkEventKey *key = (GdkEventKey *) event;
-	GdkEventButton click;
+	int y, row_local, col_local;
+	GtkAdjustment *vadj;
+
 	switch (key->keyval) {
 	case GDK_Page_Down:
-		gtk_adjustment_set_value(
-			gtk_layout_get_vadjustment (GTK_LAYOUT (et->table_canvas)),
-			CLAMP(gtk_layout_get_vadjustment (GTK_LAYOUT (et->table_canvas))->value +
-			      (gtk_layout_get_vadjustment (GTK_LAYOUT (et->table_canvas))->page_size - 20),
-			      0,
-			      gtk_layout_get_vadjustment (GTK_LAYOUT (et->table_canvas))->upper -
-			      gtk_layout_get_vadjustment (GTK_LAYOUT (et->table_canvas))->page_size));
-		click.type = GDK_BUTTON_PRESS;
-		click.window = GTK_LAYOUT (et->table_canvas)->bin_window;
-		click.send_event = key->send_event;
-		click.time = key->time;
-		click.x = 30;
-		click.y = gtk_layout_get_vadjustment (GTK_LAYOUT (et->table_canvas))->page_size - 1;
-		click.state = key->state;
-		click.button = 1;
-		gtk_widget_event(GTK_WIDGET(et->table_canvas),
-				 (GdkEvent *) &click);
+	case GDK_KP_Page_Down:
+		vadj = gtk_layout_get_vadjustment (GTK_LAYOUT (et->table_canvas));
+		y = CLAMP(vadj->value + (2 * vadj->page_size - 50), 0, vadj->upper);
+		y -= vadj->value;
+		e_table_get_cell_at (et, 30, y, &row_local, &col_local);
+		row_local = e_table_view_to_model_row (et, row_local);
+		col_local = e_selection_model_cursor_col (E_SELECTION_MODEL (et->selection));
+		e_selection_model_select_as_key_press (E_SELECTION_MODEL (et->selection), row_local, col_local, key->state);
 		return_val = 1;
 		break;
 	case GDK_Page_Up:
-		gtk_adjustment_set_value(
-			gtk_layout_get_vadjustment (GTK_LAYOUT (et->table_canvas)),
-			gtk_layout_get_vadjustment (GTK_LAYOUT (et->table_canvas))->value -
-			(gtk_layout_get_vadjustment (GTK_LAYOUT (et->table_canvas))->page_size - 20));
-		click.type = GDK_BUTTON_PRESS;
-		click.window = GTK_LAYOUT (et->table_canvas)->bin_window;
-		click.send_event = key->send_event;
-		click.time = key->time;
-		click.x = 30;
-		click.y = 1;
-		click.state = key->state;
-		click.button = 1;
-		gtk_widget_event(GTK_WIDGET(et->table_canvas),
-				 (GdkEvent *) &click);
+	case GDK_KP_Page_Up:
+		vadj = gtk_layout_get_vadjustment (GTK_LAYOUT (et->table_canvas));
+		y = CLAMP(vadj->value - (vadj->page_size - 50), 0, vadj->upper);
+		y -= vadj->value;
+		e_table_get_cell_at (et, 30, y, &row_local, &col_local);
+		row_local = e_table_view_to_model_row (et, row_local);
+		col_local = e_selection_model_cursor_col (E_SELECTION_MODEL (et->selection));
+		e_selection_model_select_as_key_press (E_SELECTION_MODEL (et->selection), row_local, col_local, key->state);
 		return_val = 1;
 		break;
 	default:
@@ -520,12 +525,25 @@ group_key_press (ETableGroup *etg, int row, int col, GdkEvent *event, ETable *et
 	return return_val;
 }
 
+static gint
+group_start_drag (ETableGroup *etg, int row, int col, GdkEvent *event, ETable *et)
+{
+	int return_val = 0;
+	gtk_signal_emit (GTK_OBJECT (et),
+			 et_signals [KEY_PRESS],
+			 row, col, event, &return_val);
+	return return_val;
+}
+
 static void
 et_table_model_changed (ETableModel *model, ETable *et)
 {
 	et->need_rebuild = TRUE;
-	if (!et->rebuild_idle_id)
+	if (!et->rebuild_idle_id) {
+		gtk_object_destroy (GTK_OBJECT (et->group));
+		et->group = NULL;
 		et->rebuild_idle_id = g_idle_add_full (20, changed_idle, et, NULL);
+	}
 }
 
 static void
@@ -555,7 +573,7 @@ et_table_rows_inserted (ETableModel *table_model, int row, int count, ETable *et
 		if (row != row_count - count)
 			e_table_group_increment(et->group, row, count);
 		for (i = 0; i < count; i++)
-			e_table_group_add (et->group, row);
+			e_table_group_add (et->group, row + i);
 		if (et->horizontal_scrolling)
 			e_table_header_update_horizontal(et->header);
 	}
@@ -568,7 +586,7 @@ et_table_rows_deleted (ETableModel *table_model, int row, int count, ETable *et)
 	if (!et->need_rebuild) {
 		int i;
 		for (i = 0; i < count; i++)
-			e_table_group_remove (et->group, row);
+			e_table_group_remove (et->group, row + i);
 		if (row != row_count)
 			e_table_group_decrement(et->group, row, count);
 		if (et->horizontal_scrolling)
@@ -589,13 +607,20 @@ et_build_groups (ETable *et)
 				       et->model,
 				       et->sort_info,
 				       0);
-	e_canvas_vbox_add_item(E_CANVAS_VBOX(et->canvas_vbox), GNOME_CANVAS_ITEM(et->group));
+	
+	if (et->use_click_to_add_end)
+		e_canvas_vbox_add_item_start(E_CANVAS_VBOX(et->canvas_vbox), GNOME_CANVAS_ITEM(et->group));
+	else
+		e_canvas_vbox_add_item(E_CANVAS_VBOX(et->canvas_vbox), GNOME_CANVAS_ITEM(et->group));
+
 	gnome_canvas_item_set(GNOME_CANVAS_ITEM(et->group),
+			      "alternating_row_colors", et->alternating_row_colors,
 			      "horizontal_draw_grid", et->horizontal_draw_grid,
 			      "vertical_draw_grid", et->vertical_draw_grid,
 			      "drawfocus", et->draw_focus,
 			      "cursor_mode", et->cursor_mode,
 			      "length_threshold", et->length_threshold,
+			      "uniform_row_height", et->uniform_row_height,
 			      "selection_model", et->selection,
 			      NULL);
 
@@ -611,6 +636,8 @@ et_build_groups (ETable *et)
 			    GTK_SIGNAL_FUNC (group_click), et);
 	gtk_signal_connect (GTK_OBJECT (et->group), "key_press",
 			    GTK_SIGNAL_FUNC (group_key_press), et);
+	gtk_signal_connect (GTK_OBJECT (et->group), "start_drag",
+			    GTK_SIGNAL_FUNC (group_start_drag), et);
 
 
 	if (!(et->is_grouped) && was_grouped)
@@ -644,7 +671,8 @@ changed_idle (gpointer data)
 	ETable *et = E_TABLE (data);
 
 	if (et->need_rebuild) {
-		gtk_object_destroy (GTK_OBJECT (et->group));
+		if (et->group)
+			gtk_object_destroy (GTK_OBJECT (et->group));
 		et_build_groups(et);
 		gtk_object_set (GTK_OBJECT (et->canvas_vbox),
 				"width", (double) GTK_WIDGET (et->table_canvas)->allocation.width,
@@ -672,8 +700,8 @@ et_canvas_realize (GtkWidget *canvas, ETable *e_table)
 		NULL);
 }
 
-static void
-et_canvas_button_press (GtkWidget *canvas, GdkEvent *event, ETable *e_table)
+static gint
+et_canvas_button_press (GtkWidget *canvas, GdkEventButton *event, ETable *e_table)
 {
 	if (GTK_WIDGET_HAS_FOCUS(canvas)) {
 		GnomeCanvasItem *item = GNOME_CANVAS(canvas)->focused_item;
@@ -682,6 +710,8 @@ et_canvas_button_press (GtkWidget *canvas, GdkEvent *event, ETable *e_table)
 			e_table_item_leave_edit(E_TABLE_ITEM(item));
 		}
 	}
+
+	return FALSE;
 }
 
 static void
@@ -752,6 +782,8 @@ e_table_setup_table (ETable *e_table, ETableHeader *full_header, ETableHeader *h
 		"spacing", 10.0,
 		NULL);
 
+	et_build_groups(e_table);
+
 	if (e_table->use_click_to_add) {
 		e_table->click_to_add = gnome_canvas_item_new (
 			GNOME_CANVAS_GROUP(e_table->canvas_vbox),
@@ -761,15 +793,19 @@ e_table_setup_table (ETable *e_table, ETableHeader *full_header, ETableHeader *h
 			"message", e_table->click_to_add_message,
 			NULL);
 
-		e_canvas_vbox_add_item (
-			E_CANVAS_VBOX(e_table->canvas_vbox),
-			e_table->click_to_add);
+		if (e_table->use_click_to_add_end)
+			e_canvas_vbox_add_item (
+				E_CANVAS_VBOX(e_table->canvas_vbox),
+				e_table->click_to_add);
+		else
+			e_canvas_vbox_add_item_start (
+				E_CANVAS_VBOX(e_table->canvas_vbox),
+				e_table->click_to_add);
+
 		gtk_signal_connect (
 			GTK_OBJECT (e_table->click_to_add), "cursor_change",
 			GTK_SIGNAL_FUNC(click_to_add_cursor_change), e_table);
 	}
-
-	et_build_groups(e_table);
 }
 
 static void
@@ -966,7 +1002,9 @@ et_real_construct (ETable *e_table, ETableModel *etm, ETableExtras *ete,
 		ete = e_table_extras_new();
 
 	e_table->use_click_to_add = specification->click_to_add;
+	e_table->use_click_to_add_end = specification->click_to_add_end;
 	e_table->click_to_add_message = e_utf8_from_locale_string (gettext (specification->click_to_add_message));
+	e_table->alternating_row_colors = specification->alternating_row_colors;
 	e_table->horizontal_draw_grid = specification->horizontal_draw_grid;
 	e_table->vertical_draw_grid = specification->vertical_draw_grid;
 	e_table->draw_focus = specification->draw_focus;
@@ -1004,6 +1042,7 @@ et_real_construct (ETable *e_table, ETableModel *etm, ETableExtras *ete,
 	gtk_object_set (GTK_OBJECT (e_table->selection),
 			"model", etm,
 			"sorter", e_table->sorter,
+			"header", e_table->header,
 			NULL);
 
 	gtk_signal_connect(GTK_OBJECT(e_table->selection), "selection_changed",
@@ -1017,6 +1056,8 @@ et_real_construct (ETable *e_table, ETableModel *etm, ETableExtras *ete,
 
 	gtk_layout_get_vadjustment (GTK_LAYOUT (e_table->table_canvas))->step_increment = 20;
 	gtk_adjustment_changed(gtk_layout_get_vadjustment (GTK_LAYOUT (e_table->table_canvas)));
+	gtk_layout_get_hadjustment (GTK_LAYOUT (e_table->table_canvas))->step_increment = 20;
+	gtk_adjustment_changed(gtk_layout_get_hadjustment (GTK_LAYOUT (e_table->table_canvas)));
 
 	if (!specification->no_headers) {
 		/*
@@ -1371,10 +1412,24 @@ e_table_get_printable (ETable *e_table)
 	return e_table_group_get_printable(e_table->group);
 }
 
+void
+e_table_right_click_up (ETable *table)
+{
+	e_selection_model_right_click_up(E_SELECTION_MODEL(table->selection));
+}
+
 static void
 et_get_arg (GtkObject *o, GtkArg *arg, guint arg_id)
 {
+	ETable *etable = E_TABLE (o);
+
 	switch (arg_id){
+	case ARG_MODEL:
+		GTK_VALUE_OBJECT (*arg) = (GtkObject *) etable->model;
+		break;
+	case ARG_UNIFORM_ROW_HEIGHT:
+		GTK_VALUE_BOOL (*arg) = etable->uniform_row_height;
+		break;
 	default:
 		break;
 	}
@@ -1399,7 +1454,14 @@ et_set_arg (GtkObject *o, GtkArg *arg, guint arg_id)
 					       NULL);
 		}
 		break;
-
+	case ARG_UNIFORM_ROW_HEIGHT:
+		etable->uniform_row_height = GTK_VALUE_BOOL (*arg);
+		if (etable->group) {
+			gnome_canvas_item_set (GNOME_CANVAS_ITEM(etable->group),
+					       "uniform_row_height", GTK_VALUE_BOOL (*arg),
+					       NULL);
+		}
+		break;
 	}
 }
 
@@ -1412,6 +1474,10 @@ set_scroll_adjustments   (ETable *table,
 		vadjustment->step_increment = 20;
 		gtk_adjustment_changed(vadjustment);
 	}
+	if (hadjustment != NULL) {
+		hadjustment->step_increment = 20;
+		gtk_adjustment_changed(hadjustment);
+	}
 
 	gtk_layout_set_hadjustment (GTK_LAYOUT(table->table_canvas),
 				    hadjustment);
@@ -1519,6 +1585,49 @@ e_table_get_cell_at (ETable *table,
 	e_table_group_compute_location(table->group, &x, &y, row_return, col_return);
 }
 
+/**
+ * e_table_get_cell_geometry:
+ * @table: The table.
+ * @row: The row to get the geometry of.
+ * @col: The col to get the geometry of.
+ * @x_return: Returns the x coordinate of the upper right hand corner of the cell with respect to the widget.
+ * @y_return: Returns the y coordinate of the upper right hand corner of the cell with respect to the widget.
+ * @width_return: Returns the width of the cell.
+ * @height_return: Returns the height of the cell.
+ * 
+ * Computes the data about this cell.
+ **/
+void
+e_table_get_cell_geometry (ETable *table,
+			   int row, int col,
+			   int *x_return, int *y_return,
+			   int *width_return, int *height_return)
+{
+	g_return_if_fail (table != NULL);
+	g_return_if_fail (E_IS_TABLE (table));
+
+	/* FIXME it would be nice if it could handle a NULL row_return or
+	 * col_return gracefully.  */
+
+	e_table_group_get_cell_geometry(table->group, &row, &col, x_return, y_return, width_return, height_return);
+
+	if (x_return)
+		(*x_return) -= GTK_LAYOUT(table->table_canvas)->hadjustment->value;
+	if (y_return) {
+		(*y_return) -= GTK_LAYOUT(table->table_canvas)->vadjustment->value;
+		(*y_return) += GTK_WIDGET(table->header_canvas)->allocation.height;
+	}
+}
+
+ESelectionModel *
+e_table_get_selection_model (ETable *table)
+{
+	g_return_val_if_fail (table != NULL, NULL);
+	g_return_val_if_fail (E_IS_TABLE (table), NULL);
+
+	return E_SELECTION_MODEL (table->selection);
+}
+
 struct _ETableDragSourceSite
 {
 	GdkModifierType    start_button_mask;
@@ -1623,6 +1732,39 @@ e_table_drag_highlight (ETable *table,
 {
 	g_return_if_fail(table != NULL);
 	g_return_if_fail(E_IS_TABLE(table));
+
+	if (row != -1) {
+		int x, y, width, height;
+		if (col == -1) {
+			e_table_get_cell_geometry (table, row, 0, &x, &y, &width, &height);
+			x = 0;
+			width = GTK_WIDGET (table->table_canvas)->allocation.width;
+		} else {
+			e_table_get_cell_geometry (table, row, col, &x, &y, &width, &height);
+			x += GTK_LAYOUT(table->table_canvas)->hadjustment->value;
+		}
+		y += GTK_LAYOUT(table->table_canvas)->vadjustment->value;
+
+		if (table->drop_highlight == NULL) {
+			table->drop_highlight =
+				gnome_canvas_item_new (gnome_canvas_root (table->table_canvas),
+						       gnome_canvas_rect_get_type (),
+						       "fill_color", NULL,
+						       /*						       "outline_color", "black",
+						       "width_pixels", 1,*/
+						       "outline_color_gdk", &(GTK_WIDGET (table)->style->fg[GTK_STATE_NORMAL]),
+						       NULL);
+		}
+		gnome_canvas_item_set (table->drop_highlight,
+				       "x1", (double) x,
+				       "x2", (double) x + width - 1,
+				       "y1", (double) y,
+				       "y2", (double) y + height - 1,
+				       NULL);
+	} else {
+		gtk_object_destroy (GTK_OBJECT (table->drop_highlight));
+		table->drop_highlight = NULL;
+	}
 }
 
 void
@@ -1630,6 +1772,11 @@ e_table_drag_unhighlight (ETable *table)
 {
 	g_return_if_fail(table != NULL);
 	g_return_if_fail(E_IS_TABLE(table));
+
+	if (table->drop_highlight) {
+		gtk_object_destroy (GTK_OBJECT (table->drop_highlight));
+		table->drop_highlight = NULL;
+	}
 }
 
 void e_table_drag_dest_set   (ETable               *table,
@@ -1678,6 +1825,40 @@ e_table_drag_dest_unset (GtkWidget *widget)
 
 /* Source side */
 
+static gint
+et_real_start_drag (ETable *table, int row, int col, GdkEvent *event)
+{
+	GtkDragSourceInfo *info;
+	GdkDragContext *context;
+	ETableDragSourceSite *site;
+
+	if (table->do_drag) {
+		site = table->site;
+
+		site->state = 0;
+		context = e_table_drag_begin (table, row, col,
+					      site->target_list,
+					      site->actions,
+					      1, event);
+
+		if (context) {
+			info = g_dataset_get_data (context, "gtk-info");
+
+			if (info && !info->icon_window) {
+				if (site->pixmap)
+					gtk_drag_set_icon_pixmap (context,
+								  site->colormap,
+								  site->pixmap,
+								  site->mask, -2, -2);
+				else
+					gtk_drag_set_icon_default (context);
+			}
+		}
+		return TRUE;
+	}
+	return FALSE;
+}
+
 void
 e_table_drag_source_set  (ETable               *table,
 			  GdkModifierType       start_button_mask,
@@ -1699,21 +1880,13 @@ e_table_drag_source_set  (ETable               *table,
 			       GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK |
 			       GDK_BUTTON_MOTION_MASK | GDK_STRUCTURE_MASK);
 
+	table->do_drag = TRUE;
+
 	if (site) {
 		if (site->target_list)
 			gtk_target_list_unref (site->target_list);
 	} else {
 		site = g_new0 (ETableDragSourceSite, 1);
-
-		table->drag_source_button_press_event_id =
-			gtk_signal_connect (GTK_OBJECT (canvas), "button_press_event",
-					    GTK_SIGNAL_FUNC (e_table_drag_source_event_cb),
-					    table);
-		table->drag_source_motion_notify_event_id =
-			gtk_signal_connect (GTK_OBJECT (canvas), "motion_notify_event",
-					    GTK_SIGNAL_FUNC (e_table_drag_source_event_cb),
-					    table);
-
 		table->site = site;
 	}
 
@@ -1738,15 +1911,10 @@ e_table_drag_source_unset (ETable *table)
 	site = table->site;
 
 	if (site) {
-		gtk_signal_disconnect (
-			GTK_OBJECT (table->table_canvas),
-			table->drag_source_button_press_event_id);
-		gtk_signal_disconnect (
-			GTK_OBJECT (table->table_canvas),
-			table->drag_source_motion_notify_event_id);
 		g_free (site);
 		table->site = NULL;
 	}
+	table->do_drag = FALSE;
 }
 
 /* There probably should be functions for setting the targets
@@ -1829,6 +1997,118 @@ et_drag_data_delete(GtkWidget *widget,
 			 context);
 }
 
+static gboolean
+do_drag_motion(ETable *et,
+	       GdkDragContext *context,
+	       gint x,
+	       gint y,
+	       guint time)
+{
+	gboolean ret_val;
+	int row, col;
+	GtkWidget *widget;
+
+	widget = GTK_WIDGET (et);
+
+	x -= widget->allocation.x;
+	y -= widget->allocation.y;
+
+	e_table_get_cell_at (et, x, y, &row, &col);
+
+	if (row != et->drop_row && col != et->drop_row) {
+		gtk_signal_emit (GTK_OBJECT (et),
+				 et_signals [TABLE_DRAG_LEAVE],
+				 et->drop_row,
+				 et->drop_col,
+				 context,
+				 time);
+	}
+	et->drop_row = row;
+	et->drop_col = col;
+	gtk_signal_emit (GTK_OBJECT (et),
+			 et_signals [TABLE_DRAG_MOTION],
+			 et->drop_row,
+			 et->drop_col,
+			 context,
+			 x,
+			 y,
+			 time,
+			 &ret_val);
+
+	return ret_val;
+}
+
+static gboolean
+scroll_timeout (gpointer data)
+{
+	ETable *et = data;
+	int dy;
+	GtkAdjustment *v;
+	double value;
+
+	if (et->scroll_down)
+		dy = 20;
+	else
+		dy = -20;
+
+	v = GTK_LAYOUT(et->table_canvas)->vadjustment;
+
+	value = v->value;
+
+	gtk_adjustment_set_value(v, CLAMP(v->value + dy, v->lower, v->upper - v->page_size));
+
+	if (v->value != value)
+		do_drag_motion(et,
+			       et->last_drop_context,
+			       et->last_drop_x,
+			       et->last_drop_y,
+			       et->last_drop_time);
+			       
+
+	return TRUE;
+}
+
+static void
+scroll_on (ETable *et, gboolean down)
+{
+	if (et->scroll_idle_id == 0 || down != et->scroll_down) {
+		if (et->scroll_idle_id != 0)
+			g_source_remove (et->scroll_idle_id);
+		et->scroll_down = down;
+		et->scroll_idle_id = g_timeout_add (100, scroll_timeout, et);
+	}
+}
+
+static void
+scroll_off (ETable *et)
+{
+	if (et->scroll_idle_id) {
+		g_source_remove (et->scroll_idle_id);
+		et->scroll_idle_id = 0;
+	}
+}
+
+static void
+context_destroyed (gpointer data)
+{
+	ETable *et = data;
+	et->last_drop_x       = 0;
+	et->last_drop_y       = 0;
+	et->last_drop_time    = 0;
+	et->last_drop_context = NULL;
+	scroll_off (et);
+	gtk_object_unref (GTK_OBJECT (et));
+}
+
+static void
+context_connect (ETable *et, GdkDragContext *context)
+{
+	if (g_dataset_get_data (context, "e-table") == NULL) {
+		gtk_object_ref (GTK_OBJECT (et));
+		g_dataset_set_data_full (context, "e-table", et, context_destroyed);
+	}
+}
+
 static void
 et_drag_leave(GtkWidget *widget,
 	      GdkDragContext *context,
@@ -1843,6 +2123,8 @@ et_drag_leave(GtkWidget *widget,
 			 time);
 	et->drop_row = -1;
 	et->drop_col = -1;
+
+	scroll_off (et);
 }
 
 static gboolean
@@ -1854,31 +2136,32 @@ et_drag_motion(GtkWidget *widget,
 	       ETable *et)
 {
 	gboolean ret_val;
-	int row, col;
 
-	e_table_get_cell_at (et, x, y, &row, &col);
+	et->last_drop_x = x;
+	et->last_drop_y = y;
+	et->last_drop_time = time;
+	et->last_drop_context = context;
+	context_connect (et, context);
 
-	if (et->drop_row >= 0 && et->drop_col >= 0 &&
-	    row != et->drop_row && col != et->drop_row) {
-		gtk_signal_emit (GTK_OBJECT (et),
-				 et_signals [TABLE_DRAG_LEAVE],
-				 et->drop_row,
-				 et->drop_col,
-				 context,
-				 time);
+	ret_val = do_drag_motion (et,
+				  context,
+				  x,
+				  y,
+				  time);
+
+
+	x -= widget->allocation.x;
+	y -= widget->allocation.y;
+
+	if (y < 20 || y > widget->allocation.height - 20) {
+		if (y < 20)
+			scroll_on (et, FALSE);
+		else
+			scroll_on (et, TRUE);
+	} else {
+		scroll_off (et);
 	}
-	et->drop_row = row;
-	et->drop_col = col;
-	if (row >= 0 && col >= 0)
-		gtk_signal_emit (GTK_OBJECT (et),
-				 et_signals [TABLE_DRAG_MOTION],
-				 et->drop_row,
-				 et->drop_col,
-				 context,
-				 x,
-				 y,
-				 time,
-				 &ret_val);
+
 	return ret_val;
 }
 
@@ -1893,41 +2176,44 @@ et_drag_drop(GtkWidget *widget,
 	gboolean ret_val;
 	int row, col;
 
+	x -= widget->allocation.x;
+	y -= widget->allocation.y;
+
 	e_table_get_cell_at (et, x, y, &row, &col);
 
-	if (et->drop_row >= 0 && et->drop_col >= 0 &&
-	    row != et->drop_row && col != et->drop_row) {
+	if (row != et->drop_row && col != et->drop_row) {
 		gtk_signal_emit (GTK_OBJECT (et),
 				 et_signals [TABLE_DRAG_LEAVE],
 				 et->drop_row,
 				 et->drop_col,
 				 context,
 				 time);
-		if (row >= 0 && col >= 0)
-			gtk_signal_emit (GTK_OBJECT (et),
-					 et_signals [TABLE_DRAG_MOTION],
-					 row,
-					 col,
-					 context,
-					 x,
-					 y,
-					 time,
-					 &ret_val);
-	}
-	et->drop_row = row;
-	et->drop_col = col;
-	if (row >= 0 && col >= 0)
 		gtk_signal_emit (GTK_OBJECT (et),
-				 et_signals [TABLE_DRAG_DROP],
-				 et->drop_row,
-				 et->drop_col,
+				 et_signals [TABLE_DRAG_MOTION],
+				 row,
+				 col,
 				 context,
 				 x,
 				 y,
 				 time,
 				 &ret_val);
+	}
+	et->drop_row = row;
+	et->drop_col = col;
+	gtk_signal_emit (GTK_OBJECT (et),
+			 et_signals [TABLE_DRAG_DROP],
+			 et->drop_row,
+			 et->drop_col,
+			 context,
+			 x,
+			 y,
+			 time,
+			 &ret_val);
 	et->drop_row = -1;
 	et->drop_col = -1;
+
+	scroll_off (et);
+
 	return ret_val;
 }
 
@@ -1943,6 +2229,9 @@ et_drag_data_received(GtkWidget *widget,
 {
 	int row, col;
 
+	x -= widget->allocation.x;
+	y -= widget->allocation.y;
+
 	e_table_get_cell_at (et, x, y, &row, &col);
 
 	gtk_signal_emit (GTK_OBJECT (et),
@@ -1957,82 +2246,6 @@ et_drag_data_received(GtkWidget *widget,
 			 time);
 }
 
-static gint
-e_table_drag_source_event_cb (GtkWidget      *widget,
-			      GdkEvent       *event,
-			      ETable         *table)
-{
-	ETableDragSourceSite *site;
-	site = table->site;
-
-	switch (event->type) {
-	case GDK_BUTTON_PRESS:
-		if ((GDK_BUTTON1_MASK << (event->button.button - 1)) & site->start_button_mask) {
-			int row, col;
-			e_table_get_cell_at (table, event->button.x, event->button.y, &row, &col);
-			if (row >= 0 && col >= 0) {
-				site->state |= (GDK_BUTTON1_MASK << (event->button.button - 1));
-				site->x = event->button.x;
-				site->y = event->button.y;
-				site->row = row;
-				site->col = col;
-			}
-		}
-		break;
-
-	case GDK_BUTTON_RELEASE:
-		if ((GDK_BUTTON1_MASK << (event->button.button - 1)) & site->start_button_mask) {
-			site->state &= ~(GDK_BUTTON1_MASK << (event->button.button - 1));
-		}
-		break;
-
-	case GDK_MOTION_NOTIFY:
-		if (site->state & event->motion.state & site->start_button_mask) {
-			/* FIXME: This is really broken and can leave us
-			 * with a stuck grab
-			 */
-			int i;
-			for (i=1; i<6; i++) {
-				if (site->state & event->motion.state &
-				    GDK_BUTTON1_MASK << (i - 1))
-					break;
-			}
-
-			if (MAX (abs (site->x - event->motion.x),
-				 abs (site->y - event->motion.y)) > 3) {
-				GtkDragSourceInfo *info;
-				GdkDragContext *context;
-
-				site->state = 0;
-				context = e_table_drag_begin (table, site->row, site->col,
-							      site->target_list,
-							      site->actions,
-							      i, event);
-
-
-				info = g_dataset_get_data (context, "gtk-info");
-
-				if (!info->icon_window) {
-					if (site->pixmap)
-						gtk_drag_set_icon_pixmap (context,
-									  site->colormap,
-									  site->pixmap,
-									  site->mask, -2, -2);
-					else
-						gtk_drag_set_icon_default (context);
-				}
-
-				return TRUE;
-			}
-		}
-		break;
-
-	default:			/* hit for 2/3BUTTON_PRESS */
-		break;
-	}
-	return FALSE;
-}
-
 static void
 e_table_class_init (ETableClass *class)
 {
@@ -2055,12 +2268,13 @@ e_table_class_init (ETableClass *class)
 	container_class->focus = et_focus;
 
 	class->cursor_change            = NULL;
-	class->cursor_activated            = NULL;
+	class->cursor_activated         = NULL;
 	class->selection_change         = NULL;
 	class->double_click             = NULL;
 	class->right_click              = NULL;
 	class->click                    = NULL;
 	class->key_press                = NULL;
+	class->start_drag               = et_real_start_drag;
 
 	class->table_drag_begin         = NULL;
 	class->table_drag_end           = NULL;
@@ -2075,7 +2289,7 @@ e_table_class_init (ETableClass *class)
 	et_signals [CURSOR_CHANGE] =
 		gtk_signal_new ("cursor_change",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETableClass, cursor_change),
 				gtk_marshal_NONE__INT,
 				GTK_TYPE_NONE, 1, GTK_TYPE_INT);
@@ -2083,7 +2297,7 @@ e_table_class_init (ETableClass *class)
 	et_signals [CURSOR_ACTIVATED] =
 		gtk_signal_new ("cursor_activated",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETableClass, cursor_activated),
 				gtk_marshal_NONE__INT,
 				GTK_TYPE_NONE, 1, GTK_TYPE_INT);
@@ -2091,7 +2305,7 @@ e_table_class_init (ETableClass *class)
 	et_signals [SELECTION_CHANGE] =
 		gtk_signal_new ("selection_change",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETableClass, selection_change),
 				gtk_marshal_NONE__NONE,
 				GTK_TYPE_NONE, 0);
@@ -2099,7 +2313,7 @@ e_table_class_init (ETableClass *class)
 	et_signals [DOUBLE_CLICK] =
 		gtk_signal_new ("double_click",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETableClass, double_click),
 				gtk_marshal_NONE__INT_INT_POINTER,
 				GTK_TYPE_NONE, 3, GTK_TYPE_INT, GTK_TYPE_INT, GTK_TYPE_GDK_EVENT);
@@ -2107,7 +2321,7 @@ e_table_class_init (ETableClass *class)
 	et_signals [RIGHT_CLICK] =
 		gtk_signal_new ("right_click",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETableClass, right_click),
 				e_marshal_INT__INT_INT_POINTER,
 				GTK_TYPE_INT, 3, GTK_TYPE_INT, GTK_TYPE_INT, GTK_TYPE_GDK_EVENT);
@@ -2115,7 +2329,7 @@ e_table_class_init (ETableClass *class)
 	et_signals [CLICK] =
 		gtk_signal_new ("click",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETableClass, click),
 				e_marshal_INT__INT_INT_POINTER,
 				GTK_TYPE_INT, 3, GTK_TYPE_INT, GTK_TYPE_INT, GTK_TYPE_GDK_EVENT);
@@ -2123,15 +2337,23 @@ e_table_class_init (ETableClass *class)
 	et_signals [KEY_PRESS] =
 		gtk_signal_new ("key_press",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETableClass, key_press),
 				e_marshal_INT__INT_INT_POINTER,
 				GTK_TYPE_INT, 3, GTK_TYPE_INT, GTK_TYPE_INT, GTK_TYPE_GDK_EVENT);
 
+	et_signals [START_DRAG] =
+		gtk_signal_new ("start_drag",
+				GTK_RUN_LAST,
+				E_OBJECT_CLASS_TYPE (object_class),
+				GTK_SIGNAL_OFFSET (ETableClass, start_drag),
+				e_marshal_INT__INT_INT_POINTER,
+				GTK_TYPE_INT, 3, GTK_TYPE_INT, GTK_TYPE_INT, GTK_TYPE_GDK_EVENT);
+
 	et_signals[TABLE_DRAG_BEGIN] =
 		gtk_signal_new ("table_drag_begin",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETableClass, table_drag_begin),
 				gtk_marshal_NONE__INT_INT_POINTER,
 				GTK_TYPE_NONE, 3,
@@ -2141,7 +2363,7 @@ e_table_class_init (ETableClass *class)
 	et_signals[TABLE_DRAG_END] =
 		gtk_signal_new ("table_drag_end",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETableClass, table_drag_end),
 				gtk_marshal_NONE__INT_INT_POINTER,
 				GTK_TYPE_NONE, 3,
@@ -2151,7 +2373,7 @@ e_table_class_init (ETableClass *class)
 	et_signals[TABLE_DRAG_DATA_GET] =
 		gtk_signal_new ("table_drag_data_get",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETableClass, table_drag_data_get),
 				e_marshal_NONE__INT_INT_POINTER_POINTER_UINT_UINT,
 				GTK_TYPE_NONE, 6,
@@ -2164,7 +2386,7 @@ e_table_class_init (ETableClass *class)
 	et_signals[TABLE_DRAG_DATA_DELETE] =
 		gtk_signal_new ("table_drag_data_delete",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETableClass, table_drag_data_delete),
 				gtk_marshal_NONE__INT_INT_POINTER,
 				GTK_TYPE_NONE, 3,
@@ -2175,7 +2397,7 @@ e_table_class_init (ETableClass *class)
 	et_signals[TABLE_DRAG_LEAVE] =
 		gtk_signal_new ("table_drag_leave",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETableClass, table_drag_leave),
 				e_marshal_NONE__INT_INT_POINTER_UINT,
 				GTK_TYPE_NONE, 4,
@@ -2186,7 +2408,7 @@ e_table_class_init (ETableClass *class)
 	et_signals[TABLE_DRAG_MOTION] =
 		gtk_signal_new ("table_drag_motion",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETableClass, table_drag_motion),
 				e_marshal_BOOL__INT_INT_POINTER_INT_INT_UINT,
 				GTK_TYPE_BOOL, 6,
@@ -2199,7 +2421,7 @@ e_table_class_init (ETableClass *class)
 	et_signals[TABLE_DRAG_DROP] =
 		gtk_signal_new ("table_drag_drop",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETableClass, table_drag_drop),
 				e_marshal_BOOL__INT_INT_POINTER_INT_INT_UINT,
 				GTK_TYPE_BOOL, 6,
@@ -2212,7 +2434,7 @@ e_table_class_init (ETableClass *class)
 	et_signals[TABLE_DRAG_DATA_RECEIVED] =
 		gtk_signal_new ("table_drag_data_received",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETableClass, table_drag_data_received),
 				e_marshal_NONE__INT_INT_POINTER_INT_INT_POINTER_UINT_UINT,
 				GTK_TYPE_NONE, 8,
@@ -2225,20 +2447,24 @@ e_table_class_init (ETableClass *class)
 				GTK_TYPE_UINT,
 				GTK_TYPE_UINT);
 
-	gtk_object_class_add_signals (object_class, et_signals, LAST_SIGNAL);
+	E_OBJECT_CLASS_ADD_SIGNALS (object_class, et_signals, LAST_SIGNAL);
 
 	class->set_scroll_adjustments = set_scroll_adjustments;
 
 	widget_class->set_scroll_adjustments_signal =
 		gtk_signal_new ("set_scroll_adjustments",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETableClass, set_scroll_adjustments),
 				gtk_marshal_NONE__POINTER_POINTER,
 				GTK_TYPE_NONE, 2, GTK_TYPE_ADJUSTMENT, GTK_TYPE_ADJUSTMENT);
 
 	gtk_object_add_arg_type ("ETable::length_threshold", GTK_TYPE_INT,
 				 GTK_ARG_WRITABLE, ARG_LENGTH_THRESHOLD);
+	gtk_object_add_arg_type ("ETable::uniform_row_height", GTK_TYPE_BOOL,
+				 GTK_ARG_READWRITE, ARG_UNIFORM_ROW_HEIGHT);
+	gtk_object_add_arg_type ("ETable::model", E_TABLE_MODEL_TYPE,
+				 GTK_ARG_READABLE, ARG_MODEL);
 }
 
 E_MAKE_TYPE(e_table, "ETable", ETable, e_table_class_init, e_table_init, PARENT_TYPE);
