@@ -13,13 +13,15 @@
  */
 
 #include <config.h>
+#include <stdlib.h>
 #include <string.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
-#include <unicode.h>
 #include <iconv.h>
 #include "gal/util/e-cache.h"
 #include "e-font.h"
+#include "e-unicode.h"
+#include "gal/util/e-iconv.h"
 
 #ifdef HAVE_CODESET
 #include <langinfo.h>
@@ -62,11 +64,7 @@ static void split_name (gchar * c[], gchar * name);
 static gboolean find_variants (gchar **namelist, gint length,
 			       gchar *base_weight, gchar **light,
 			       gchar **bold);
-static const gchar *get_locale_charset (void);
 static gint         e_font_to_native   (EFont *font, gchar *native, const gchar *utf, gint bytes);
-#if 0
-static const gchar *locale_charset_to_encoding(void);
-#endif
 #ifdef E_FONT_VERBOSE
 static void e_font_print_gdk_font_name (const GdkFont * font);
 #endif
@@ -97,6 +95,7 @@ e_font_from_gdk_name (const gchar *name)
 		need_fontset = TRUE;
 		g_snprintf (n, 1024, name);
 	} else {
+#if 0
 		gchar *c[14], *nn;
 		gint i;
 		/* Split name into pieces */
@@ -116,11 +115,14 @@ e_font_from_gdk_name (const gchar *name)
 			/* fixme: Whoever actually uses CJK locale should check/change that */
 			if (!strncasecmp ("eucjp", c[E_XF_CHARSET], len)) need_fontset = TRUE;
 			else if (!strncasecmp ("ujis", c[E_XF_CHARSET], len)) need_fontset = TRUE;
-			else if (!strncasecmp ("jisx208", c[E_XF_CHARSET], len)) need_fontset = TRUE;
+			else if (!strncasecmp ("jis", c[E_XF_CHARSET], 3)) need_fontset = TRUE;
 			else if (!strncasecmp ("gb2312", c[E_XF_CHARSET], len)) need_fontset = TRUE;
 			else if (!strncasecmp ("ksc5601", c[E_XF_CHARSET], len)) need_fontset = TRUE;
 		}
 		g_free (nn);
+#else
+		if (MB_CUR_MAX > 1) need_fontset = TRUE;
+#endif
 		if (need_fontset) {
 			/* We do not have ',', so we have to extend name */
 			g_snprintf (n, 1024, "%s,*", name);
@@ -211,6 +213,7 @@ e_font_from_gdk_font (GdkFont *gdkfont)
 	}
 
 	if (gdkfont->type == GDK_FONT_FONTSET) {
+#if 0
 		gchar *xlocale[] = {	"ja_JP.euc", "ja_JP.ujis",
 					"zh_CN.GB2312", "zh_TW.Big5",
 					"ko_KR.eucKR", NULL};
@@ -231,6 +234,18 @@ e_font_from_gdk_font (GdkFont *gdkfont)
 				return font;
 			}
 		}
+#else
+		if (MB_CUR_MAX > 1) {
+			/* We need fontset */
+			font = e_font_from_gdk_fontset (gdkfont);
+			/* Insert into cache */
+			if (e_cache_insert (cache, keyfont, font, 1)) {
+				/* Inserted, so add cache ref manually */
+				e_font_ref (font);
+			}
+			return font;
+		}
+#endif
 	}
 
 	/* We can do with simple font */
@@ -608,6 +623,8 @@ e_font_unref (EFont *font)
 	font->refcount--;
 
 	if (font->refcount < 1) {
+		e_iconv_close(font->to);
+		e_iconv_close(font->from);
 		gdk_font_unref (font->font);
 		if (font->bold) gdk_font_unref (font->bold);
 		g_free (font);
@@ -631,13 +648,13 @@ no_conv_wrapper (EFont *font, gchar *native, const gchar *utf, gint bytes)
 {
 	gint len;
 	const gchar *u;
-	unicode_char_t uc;
+	gunichar uc;
 
 	u   = utf;
 	len = 0;
 
 	while (u && u - utf < bytes) {
-		u = unicode_get_utf8 (u, &uc);
+		u = e_unicode_get_utf8 (u, &uc);
 		if (font->twobyte) {
 			native [len] = (uc & 0xff00) >> 8; len++;
 		}
@@ -714,16 +731,8 @@ e_font_to_native (EFont *font, gchar *native, const gchar *utf, gint bytes)
 		while (ibl > 0) {
 			iconv (font->to, &ib, &ibl, &ob, &obl);
 			if (ibl > 0) {
-				gint len;
-				if ((*ib & 0x80) == 0x00) len = 1;
-				else if ((*ib &0xe0) == 0xc0) len = 2;
-				else if ((*ib &0xf0) == 0xe0) len = 3;
-				else if ((*ib &0xf80) == 0xf0) len = 4;
-				else {
-					g_warning ("Invalid UTF-8 sequence");
-					break;
-				}
-				ib += len;
+				ib = g_utf8_next_char (ib);
+
 				ibl = bytes - (ib - utf);
 				if (ibl > bytes) ibl = 0;
 				if (!font->twobyte) {
@@ -761,7 +770,7 @@ e_font_draw_utf8_text (GdkDrawable *drawable, EFont *font, EFontStyle style, Gdk
 	native = alloca (numbytes * 4);
 
 	native_bytes = e_font_to_native (font, native, text, numbytes);
-
+	
 	if ((style & E_FONT_BOLD) && (font->bold)) {
 		gdk_draw_text (drawable, font->bold, gc, x, y, native, native_bytes);
 	} else {
@@ -799,140 +808,10 @@ e_font_utf8_text_width (EFont *font, EFontStyle style, const char *text, int num
 gint
 e_font_utf8_char_width (EFont *font, EFontStyle style, char *text)
 {
-	gint len;
-
 	g_return_val_if_fail (font != NULL, 0);
 	g_return_val_if_fail (text != NULL, 0);
 
-	if ((*text & 0x80) == 0x00) len = 1;
-	else if ((*text &0xe0) == 0xc0) len = 2;
-	else if ((*text &0xf0) == 0xe0) len = 3;
-	else if ((*text &0xf80) == 0xf0) len = 4;
-	else {
-		g_warning ("Invalid UTF-8 sequence");
-		return 0;
-	}
-
-	return e_font_utf8_text_width (font, style, text, len);
-}
-
-static const gchar *
-translate_encoding (const gchar *encoding)
-{
-	static GHashTable *eh = NULL;
-	gchar e[64];
-	gchar *finalenc;
-
-	if (eh == NULL) {
-		eh = g_hash_table_new (g_str_hash, g_str_equal);
-
-		g_hash_table_insert (eh, "iso8859-1", "iso-8859-1");
-		g_hash_table_insert (eh, "iso_8859-1", "iso-8859-1");
-		g_hash_table_insert (eh, "iso8859-2", "iso-8859-2");
-		g_hash_table_insert (eh, "iso_8859-2", "iso-8859-2");
-		g_hash_table_insert (eh, "iso8859-3", "iso-8859-3");
-		g_hash_table_insert (eh, "iso_8859-3", "iso-8859-3");
-		g_hash_table_insert (eh, "iso8859-4", "iso-8859-4");
-		g_hash_table_insert (eh, "iso_8859-4", "iso-8859-4");
-		g_hash_table_insert (eh, "iso8859-5", "iso-8859-5");
-		g_hash_table_insert (eh, "iso_8859-5", "iso-8859-5");
-		g_hash_table_insert (eh, "iso8859-6", "iso-8859-6");
-		g_hash_table_insert (eh, "iso_8859-6", "iso-8859-6");
-		g_hash_table_insert (eh, "iso8859-7", "iso-8859-7");
-		g_hash_table_insert (eh, "iso_8859-7", "iso-8859-7");
-		g_hash_table_insert (eh, "iso8859-8", "iso-8859-8");
-		g_hash_table_insert (eh, "iso_8859-8", "iso-8859-8");
-		g_hash_table_insert (eh, "iso8859-9", "iso-8859-9");
-		g_hash_table_insert (eh, "iso_8859-9", "iso-8859-9");
-		g_hash_table_insert (eh, "iso8859-10", "iso-8859-10");
-		g_hash_table_insert (eh, "iso_8859-10", "iso-8859-10");
-		g_hash_table_insert (eh, "iso8859-13", "iso-8859-13");
-		g_hash_table_insert (eh, "iso_8859-13", "iso-8859-13");
-		g_hash_table_insert (eh, "iso8859-14", "iso-8859-14");
-		g_hash_table_insert (eh, "iso_8859-14", "iso-8859-14");
-		g_hash_table_insert (eh, "iso8859-15", "iso-8859-15");
-		g_hash_table_insert (eh, "iso_8859-15", "iso-8859-15");
-		g_hash_table_insert (eh, "iso10646-1", "iso-10646-1");
-		g_hash_table_insert (eh, "iso_10646-1", "iso-10646-1");
-		g_hash_table_insert (eh, "koi8-r", "koi8-r");
-		g_hash_table_insert (eh, "koi8-ub", "koi8-ub");
-		g_hash_table_insert (eh, "eucjp", "EUC-JP");
-		g_hash_table_insert (eh, "ujis", "EUC-JP");
-	}
-
-	strncpy (e, encoding, 64);
-	g_strdown (e);
-
-	finalenc = g_hash_table_lookup (eh, e);
-
-	return finalenc ? finalenc : encoding;
-}
-
-static const gchar *
-get_locale_charset (void)
-{
-	gchar *locale;
-	static gchar *charset = NULL;
-	static gboolean initialized = FALSE;
-
-	if (!initialized) {
-		locale = gtk_set_locale ();
-		if ( strcmp (locale, "C") == 0 )
-			locale = NULL;
-		if ( locale != NULL ) {
-			gchar * p;
-			size_t len;
-
-			p = strchr (locale, '@');
-			len = p ? (p - locale) : strlen(locale);
-			p = strchr (locale, '.');
-			if (p) {
-				charset = g_strndup(p+1, len - (p - locale) + 1);
-				g_strdown (charset);
-			}
-		}
-		initialized = TRUE;
-	}
-
-#ifdef E_FONT_VERBOSE
-	if (e_font_verbose) {
-		g_print ("Locale charset is: %s\n", charset);
-	}
-#endif
-
-	return charset;
-}
-
-#if 0
-static const gchar *
-locale_charset_to_encoding(void)
-{
-	static GHashTable *lh = NULL;
-	const gchar *charset;
-
-	if (!lh) {
-		lh = g_hash_table_new (g_str_hash, g_str_equal);
-
-		g_hash_table_insert (lh, "eucjp", "iso8859-1");
-		g_hash_table_insert (lh, "ujis", "iso8859-1");
-	}
-	charset = get_locale_charset ();
-
-	return g_hash_table_lookup (lh, charset);
-}
-#endif
-
-const gchar *
-e_locale_encoding (void)
-{
-#ifdef HAVE_CODESET
-	const char *encoding = nl_langinfo(CODESET);
-	if (encoding) {
-		const gchar *trans_encoding = translate_encoding (encoding);
-		if (trans_encoding) return trans_encoding;
-	}
-#endif
-	return NULL;
+	return e_font_utf8_text_width (font, style, text, g_utf8_skip[*(guchar *)text]);
 }
 
 const gchar *
@@ -966,11 +845,9 @@ e_gdk_font_encoding (GdkFont *font)
 
 	if (font->type == GDK_FONT_FONTSET) {
 		/* We are fontset, so try to find right encoding from locale */
-		encoding = get_locale_charset ();
-		if (encoding) {
-			const gchar *trans_encoding = translate_encoding (encoding);
-			if (trans_encoding) return trans_encoding;
-		}
+		encoding = e_iconv_charset_name(e_iconv_locale_charset());
+		if (encoding)
+			return encoding;
 	}
 
 	font_atom = gdk_atom_intern ("FONT", FALSE);
@@ -1012,7 +889,7 @@ e_gdk_font_encoding (GdkFont *font)
 		return NULL;
 	}
 
-	encoding = translate_encoding (p);
+	encoding = e_iconv_charset_name(p);
 	e_cache_insert (cache, font, (gpointer) encoding, 1);
 	g_free (name);
 
@@ -1020,76 +897,16 @@ e_gdk_font_encoding (GdkFont *font)
 	return encoding;
 }
 
-static iconv_t
-e_iconv_from_encoding (const gchar *enc)
-{
-	static GHashTable *ih = NULL;
-	iconv_t ic;
-
-	if (!enc) return (iconv_t) -1;
-
-	if (!ih) ih = g_hash_table_new (g_str_hash, g_str_equal);
-
-	ic = g_hash_table_lookup (ih, enc);
-
-	if (!ic) {
-		ic = iconv_open ("UTF-8", enc);
-		if (ic == (iconv_t) -1) return ic;
-		g_hash_table_insert (ih, (gpointer) enc, ic);
-	}
-
-	return ic;
-}
-
-static iconv_t
-e_iconv_to_encoding (const gchar *enc)
-{
-	static GHashTable *ih = NULL;
-	iconv_t ic;
-
-	if (!enc) return (iconv_t) -1;
-
-	if (!ih) ih = g_hash_table_new (g_str_hash, g_str_equal);
-
-	ic = g_hash_table_lookup (ih, enc);
-
-	if (!ic) {
-		ic = iconv_open (enc, "UTF-8");
-		if (ic == (iconv_t) -1) return ic;
-		g_hash_table_insert (ih, (gpointer) enc, ic);
-	}
-
-	return ic;
-}
-
-iconv_t
-e_iconv_from_locale (void)
-{
-	const gchar *enc;
-
-	enc = e_locale_encoding ();
-	return e_iconv_from_encoding (enc);
-}
-
-iconv_t
-e_iconv_to_locale (void)
-{
-	const gchar *enc;
-
-	enc = e_locale_encoding ();
-	return e_iconv_to_encoding (enc);
-}
-
 iconv_t
 e_iconv_from_gdk_font (GdkFont *font)
 {
 	const gchar *enc;
 
-	if (!font) return (iconv_t) -1;
-
 	enc = e_gdk_font_encoding (font);
+	if (enc == NULL)
+		return (iconv_t) -1;
 
-	return e_iconv_from_encoding (enc);
+	return e_iconv_open("utf-8", enc);
 }
 
 iconv_t
@@ -1097,11 +914,11 @@ e_iconv_to_gdk_font (GdkFont *font)
 {
 	const gchar *enc;
 
-	if (!font) return (iconv_t) -1;
-
 	enc = e_gdk_font_encoding (font);
+	if (enc == NULL)
+		return (iconv_t) -1;
 
-	return e_iconv_to_encoding (enc);
+	return e_iconv_open(enc, "utf-8");
 }
 
 /*
@@ -1271,8 +1088,6 @@ e_font_print_gdk_font_name (const GdkFont * font)
 
 	font_atom = gdk_atom_intern ("FONT", FALSE);
 
-	g_print ("-------- start of font description --------\n");
-
 	if (font == NULL) {
 		g_print ("GdkFont is NULL\n");
 	} else if (font->type == GDK_FONT_FONTSET) {
@@ -1294,7 +1109,5 @@ e_font_print_gdk_font_name (const GdkFont * font)
 		g_print ("GdkFont: %s\n", name);
 		if (name) g_free (name);
 	}
-
-	g_print ("-------- end of font description --------\n");
 }
 #endif
