@@ -43,10 +43,6 @@ GdkPixbuf *icon_pixbufs[NUM_SHORTCUT_TYPES];
 
 GtkWidget *main_label;
 
-static GdkPixbuf *icon_callback  (EShortcutBar *shortcut_bar,
-				  const gchar  *url,
-				  gpointer      data);
-
 static void on_main_label_size_allocate (GtkWidget *widget,
 					 GtkAllocation *allocation,
 					 gpointer data);
@@ -135,9 +131,7 @@ main (int argc, char *argv[])
 				  shortcut_model);
 	e_paned_pack1 (E_PANED (hpaned), shortcut_bar, FALSE, TRUE);
 	gtk_widget_show (shortcut_bar);
-	e_shortcut_bar_set_icon_callback (E_SHORTCUT_BAR (shortcut_bar),
-					  icon_callback,
-					  NULL);
+
 #if 0
 	e_shortcut_bar_set_enable_drags (E_SHORTCUT_BAR (shortcut_bar),
 					 FALSE);
@@ -197,10 +191,13 @@ main (int argc, char *argv[])
 	/* Load our default icons. */
 	for (i = 0; i < NUM_SHORTCUT_TYPES; i++) {
 		pathname = gnome_pixmap_file (icon_filenames[i]);
-		if (pathname)
+		if (pathname) {
 			icon_pixbufs[i] = gdk_pixbuf_new_from_file (pathname);
-		else
+			g_print ("type %d -- %s %s\n",
+				 i, pathname, icon_pixbufs[i] == NULL ? "(not found)" : "");
+		} else {
 			icon_pixbufs[i] = NULL;
+		}
 	}
 
 	add_test_groups (E_SHORTCUT_BAR (shortcut_bar));
@@ -215,25 +212,6 @@ main (int argc, char *argv[])
 }
 
 
-static GdkPixbuf*
-icon_callback (EShortcutBar *shortcut_bar,
-	       const gchar *url,
-	       gpointer data)
-{
-	gint i;
-
-	for (i = 0; i < NUM_SHORTCUT_TYPES; i++) {
-		if (!strncmp (url, shortcut_types[i],
-			      strlen (shortcut_types[i]))) {
-			if (icon_pixbufs[i])
-				gdk_pixbuf_ref (icon_pixbufs[i]);
-			return icon_pixbufs[i];
-		}
-	}
-
-	return NULL;
-}
-
 static void
 on_main_label_size_allocate (GtkWidget *widget,
 			     GtkAllocation *allocation,
@@ -288,7 +266,8 @@ add_test_group (EShortcutBar *shortcut_bar, gint i, gchar *group_name)
 		item_num = e_shortcut_model_add_item (shortcut_bar->model,
 						      group_num, -1,
 						      shortcut_types[shortcut_type],
-						      buffer);
+						      buffer,
+						      icon_pixbufs [shortcut_type]);
 	}
 }
 
@@ -658,5 +637,5 @@ on_shortcut_dropped	(EShortcutBar	*shortcut_bar,
 		 item_num);
 
 	e_shortcut_model_add_item (shortcut_bar->model,
-				   group_num, item_num, url, name);
+				   group_num, item_num, url, name, NULL);
 }
